@@ -579,6 +579,18 @@ static VALUE rhrdt_s__strptime(int argc, VALUE *argv, VALUE klass) {
   return rhrd__strptime(argv[0], fmt_str, fmt_len);
 }
 
+/* call-seq:
+ *   allocate() -> DateTime <br />
+ *
+ * Returns a +DateTime+ object for julian day 0.
+ */
+static VALUE rhrdt_s_allocate(VALUE klass) {
+  rhrdt_t *d;
+  VALUE rd = Data_Make_Struct(klass, rhrdt_t, NULL, -1, d);
+  d->flags = RHR_HAVE_JD | RHR_HAVE_NANOS;
+  return rd;
+}
+
 /* call-seq:
  *   civil() -> DateTime <br />
  *   civil(year, month=1, day=1, hour=0, minute=0, second=0, offset=0, sg=nil) -> DateTime
@@ -1000,6 +1012,20 @@ static VALUE rhrdt_asctime(VALUE self) {
   RHR_RETURN_RESIZED_STR(s, len)
 }
 
+/* call-seq:
+ *   clone() -> DateTime
+ *
+ * Returns a clone of the receiver.
+ */
+static VALUE rhrdt_clone(VALUE self) {
+  rhrdt_t *d, *nd;
+  VALUE rd = rb_call_super(0, NULL);
+  Data_Get_Struct(self, rhrdt_t, d);
+  Data_Get_Struct(rd, rhrdt_t, nd);
+  memcpy(nd, d, sizeof(rhrdt_t));
+  return rd;
+}
+
 /* call-seq:
  *   cwday() -> Integer
  *
@@ -1103,6 +1129,20 @@ static VALUE rhrdt_day_fraction(VALUE self) {
   return rb_float_new((double)dt->nanos/RHR_NANOS_PER_DAYD);
 }
 
+/* call-seq:
+ *   dup() -> DateTime
+ *
+ * Returns a dup of the receiver.
+ */
+static VALUE rhrdt_dup(VALUE self) {
+  rhrdt_t *d, *nd;
+  VALUE rd = rb_call_super(0, NULL);
+  Data_Get_Struct(self, rhrdt_t, d);
+  Data_Get_Struct(rd, rhrdt_t, nd);
+  memcpy(nd, d, sizeof(rhrdt_t));
+  return rd;
+}
+
 /* call-seq:
  *   downto(target){|datetime|} -> DateTime
  *
@@ -1469,6 +1509,9 @@ static VALUE rhrdt_step(int argc, VALUE *argv, VALUE self) {
       step = NUM2DBL(rstep);
       step_jd = (long)floor(step);
       step_nanos = llround((step - step_jd)*RHR_NANOS_PER_DAY);
+      if (step_jd == 0 && step_nanos == 0) {
+        rb_raise(rb_eArgError, "step can't be 0");
+      }
       break;
     default:
       rb_raise(rb_eArgError, "wrong number of arguments: %i for 2", argc);
@@ -2741,6 +2784,7 @@ void Init_datetime(void) {
   /* Define class */
 
   rhrdt_class = rb_define_class("DateTime", rhrd_class);
+  rb_define_alloc_func(rhrdt_class, rhrdt_s_allocate);
   rhrdt_s_class = rb_singleton_class(rhrdt_class);
 
   /* Define methods for all ruby versions*/
@@ -2765,12 +2809,14 @@ void Init_datetime(void) {
   rb_define_method(rhrdt_class, "ajd", rhrdt_ajd, 0);
   rb_define_method(rhrdt_class, "amjd", rhrdt_amjd, 0);
   rb_define_method(rhrdt_class, "asctime", rhrdt_asctime, 0);
+  rb_define_method(rhrdt_class, "clone", rhrdt_clone, 0);
   rb_define_method(rhrdt_class, "cwday", rhrdt_cwday, 0);
   rb_define_method(rhrdt_class, "cweek", rhrdt_cweek, 0);
   rb_define_method(rhrdt_class, "cwyear", rhrdt_cwyear, 0);
   rb_define_method(rhrdt_class, "day", rhrdt_day, 0);
   rb_define_method(rhrdt_class, "day_fraction", rhrdt_day_fraction, 0);
   rb_define_method(rhrdt_class, "downto", rhrdt_downto, 1);
+  rb_define_method(rhrdt_class, "dup", rhrdt_dup, 0);
   rb_define_method(rhrdt_class, "eql?", rhrdt_eql_q, 1);
   rb_define_method(rhrdt_class, "hash", rhrdt_hash, 0);
   rb_define_method(rhrdt_class, "hour", rhrdt_hour, 0);
